// file: signal_detector/src_01/sigdp01_sm_1.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: sm_advance_cc
//
// arguments:
//  float_4 egy_p: (input) energy value to be considered
//
// return:
//  an int_4 value indicating the current state
//
// this method advanced the state machine by one input value. this is
// really the crucial method in this class.
//
// there are four states in this machine, and 12 types of transitions
// that can be made:
//
//  trans 00:		nois -> sign
//  trans 01:		nois -> ntos
//  trans 02:		nois -> nois
//  trans 03:		ntos -> ntos
//  trans 04:		ntos -> nois
//  trans 05:		ntos -> sign
//  trans 06:		sign -> sign
//  trans 07:		sign -> ston
//  trans 08:		sign -> nois
//  trans 09:		ston -> ston
//  trans 10:		ston -> nois
//  trans 11:		ston -> sign

// 
// Each of these is treated seperately below.
// 
int_4 Signal_detector_01::sm_advance_cc(float_4 egy_p) {

  // shift contents
  //
  states_d[2] = states_d[1];
  states_d[1] = states_d[0];

  // treat the states emanating from the noise state (00 to 02)
  //
  if (states_d[1] == SIGD_STATE_NOIS) {

    // trans #00
    //
    if (egy_p > (sig_thresh_d-utt_delta_d)) {
      states_d[0] = SIGD_STATE_SIGN;
      durations_d[SIGD_STATE_SIGN] = (int_4)1;
    }
    // trans # 01
    //
    else if (egy_p > (noise_thresh_d+utt_delta_d)) {
      states_d[0] = SIGD_STATE_NTOS;
      durations_d[SIGD_STATE_NTOS] = durations_d[SIGD_STATE_NOIS] + (int_4)1;
    }
    // trans # 02
    //
    else if (egy_p <= (noise_thresh_d+utt_delta_d)) {
      states_d[0] = SIGD_STATE_NOIS;
      durations_d[SIGD_STATE_NOIS] += (int_4)1;
    }
    // illegal transition
    //
    else {
      error_handler_cc((char_1*)"sm_advance_cc",
		       (char_1*)"invalid transition from nois");
    }
  }

  // treat the states emanating from the noise to signal state (03 to 05)
  //
  else if (states_d[1] == SIGD_STATE_NTOS) {

    // trans #03
    //
    if ((egy_p > (noise_thresh_d+utt_delta_d)) &&
	(egy_p < (sig_thresh_d-utt_delta_d))) {
      states_d[0] = SIGD_STATE_NTOS;
      durations_d[SIGD_STATE_NTOS] += (int_4)1;
    }
    // trans # 04
    //
    else if (egy_p <= (noise_thresh_d+utt_delta_d)) {
      states_d[0] = SIGD_STATE_NOIS;
      durations_d[SIGD_STATE_NOIS] = durations_d[SIGD_STATE_NTOS] + (int_4)1;
    }
    // trans # 05
    //
    else if (egy_p >= (sig_thresh_d-utt_delta_d)) {
      states_d[0] = SIGD_STATE_SIGN;
      durations_d[SIGD_STATE_SIGN] = (int_4)1;
    }
    // illegal transition
    //
    else {
      error_handler_cc((char_1*)"sm_advance_cc",
		       (char_1*)"invalid transition from ntos");
    }
  }

  // treat the states emanating from the signal state (06 to 08)
  //
  else if (states_d[1] == SIGD_STATE_SIGN) {

    // trans #06
    //
    if (egy_p >= (sig_thresh_d-utt_delta_d)) {
      states_d[0] = SIGD_STATE_SIGN;
      durations_d[SIGD_STATE_SIGN] += (int_4)1;
    }
    // trans # 07
    //
    else if ((egy_p < (sig_thresh_d-utt_delta_d)) &&
	     (egy_p > (noise_thresh_d+utt_delta_d))) {
      states_d[0] = SIGD_STATE_STON;
      durations_d[SIGD_STATE_STON] = durations_d[SIGD_STATE_SIGN] + (int_4)1;
    }
    // trans # 08
    //
    else if (egy_p <= (noise_thresh_d+utt_delta_d)) {
      states_d[0] = SIGD_STATE_NOIS;
      durations_d[SIGD_STATE_NOIS] = (int_4)1;
    }
    // illegal transition
    //
    else {
      error_handler_cc((char_1*)"sm_advance_cc",
		       (char_1*)"invalid transition from sign");
    }
  }

  // treat the states emanating from the signal to noise state (09 to 11)
  //
  else if (states_d[1] == SIGD_STATE_STON) {

    // trans #09
    //
    if ((egy_p < (sig_thresh_d-utt_delta_d)) &&
	(egy_p > (noise_thresh_d+utt_delta_d))) {
      states_d[0] = SIGD_STATE_STON;
      durations_d[SIGD_STATE_STON] += (int_4)1;
    }
    // trans # 10
    //
    else if (egy_p <= (noise_thresh_d+utt_delta_d)) {
      states_d[0] = SIGD_STATE_NOIS;
      durations_d[SIGD_STATE_NOIS] = (int_4)1;
    }
    // trans # 11
    //
    else if (egy_p >= (sig_thresh_d-utt_delta_d)) {
      states_d[0] = SIGD_STATE_SIGN;
      durations_d[SIGD_STATE_SIGN] = durations_d[SIGD_STATE_STON] + (int_4)1;
    }
    // illegal transition
    //
    else {
      error_handler_cc((char_1*)"sm_advance_cc",
		       (char_1*)"invalid transition from ston");
    }
  }

  // exit gracefully
  //
  return states_d[0];
}
